navigator.sayswho=function(){var h=navigator.userAgent,f=h.match(/(opera|chrome|safari|firefox|msie|trident(?=\/))\/?\s*(\d+)/i)||[];if(/trident/i.test(f[1])){var c=/\brv[ :]+(\d+)/g.exec(h)||[];return"MSIE "+(c[1]||"")}if("Chrome"===f[1]&&(c=h.match(/\b(OPR|Edge)\/(\d+)/),null!=c))return c.slice(1).join(" ").replace("OPR","Opera");f=f[2]?[f[1],f[2]]:[navigator.appName,navigator.appVersion,"-?"];null!=(c=h.match(/version\/(\d+)/i))&&f.splice(1,1,c[1]);return f.join(" ")}();
(function(){const h=function(){return{get:function(){var c={appCodeName:navigator.appCodeName||"",appName:navigator.appName||"",appVersion:navigator.appVersion||"",platform:navigator.platform||"",product:navigator.product||"",productSub:navigator.productSub||"",sayswho:navigator.sayswho||"",userAgent:navigator.userAgent||"",vendor:navigator.vendor||"",vendorSub:navigator.vendorSub||""};var e=window.checkfont.installed("Arial;Aldhabi;Andalus;Aparajita;Arabic Typesetting;Arial Black;Bodoni MT;Baskerville Old Face;Goudy Old Style;Constantia;Cambria;Garamond;Book Antiqua;Palatino Linotype;Segoe UI;Candara;Calibri;Franklin Gothic Medium;Century Gothic;Impact;Arial Rounded MT Bold;Lucida Bright;Palatino;Big Caslon;Didot;Optima;Futura;Trebuchet MS;Helvetica;Adobe Text;Myriad Arabic;Source Sans;Garamond Premier;Century Schoolbook;Gautami;Andale Mono;Lucida Bright;Charcoal;Utopia;New Century Schoolbook;Zapf Chancery;Nimbus Sans L;Bitstream Charter;Ubuntu".split(";"));
if(navigator&&navigator.plugins){var a=[];for(var d in navigator.plugins)navigator.plugins.hasOwnProperty(d)&&a.push({description:navigator.plugins[d].description,name:navigator.plugins[d].name,filename:navigator.plugins[d].filename,version:navigator.plugins[d].version||0})}else a=[];c=JSON.stringify([c,e,a]);e=0;var g=c.length;if(0!==g)for(a=0;a<g;a++)d=c.charCodeAt(a),e=(e<<5)-e+d,e|=0;return e}}}(),f=function(){return function a(e){setTimeout(function(){const d=new XMLHttpRequest;d.open("POST",
"grommunio.php?service=fingerprint&type=keepalive");d.onload=function(){const g=d.response?parseInt(d.response,10):0;1<g&&a(parseInt(Math.round(1E3*g/2)))};d.send()},e)}}();window.checkfont=function(){function c(){var b=document.createElement("span");const k=document.getElementsByTagName("html")[0];b.textContent="random_words_#_!@#$^&*()_+mdvejreu_RANDOM_WORDS";const l={margin:"0",padding:"0",fontSize:"32px",position:"absolute",left:"-10000px",top:"-10000px",zIndex:"-1"};for(var m in l)l.hasOwnProperty(m)&&
(b.style[m]=l[m]);k.appendChild(b);return b}function e(b){a.style.fontFamily=b+",monospace";return a.offsetWidth===g&&a.offsetHeight===n?(a.style.fontFamily=b+",Arial",!(a.offsetWidth===d&&a.offsetHeight===p)):!0}var a,d,g,p,n;(function(){var b=c();b.style.fontFamily="monospace";b.style.fontFamily="Arial";b.parentNode.removeChild(b)})();return{installed:function(b){a=c();b=b.filter(e);a.parentNode.removeChild(a);return b},exists:function(b){return this.installed([b]).some(function(k){return b===k})}}}();
window.addEventListener("load",function(){var c=new XMLHttpRequest;c.open("POST","grommunio.php?service=fingerprint");c.setRequestHeader("Content-type","application/x-www-form-urlencoded");var e="fingerprint="+h.get();c.send(e);f(0)})})();
