body{font-family:Verdana,Arial,Helvetica,sans-serif;font-size:11px;background:#f0f0ee;padding:0;margin:8px 8px 0 8px}html{background:#f0f0ee}td{font-family:Verdana,Arial,Helvetica,sans-serif;font-size:10px}textarea{resize:none;outline:0}a:link,a:visited{color:black}a:hover{color:#2b6fb6}.nowrap{white-space:nowrap}fieldset{margin:0;padding:4px;border:1px solid #919b9c;font-family:Verdana,Arial;font-size:10px}legend{color:#2b6fb6;font-weight:bold}label.msg{display:none}label.invalid{color:#e00;display:inline}input.invalid{border:1px solid #e00}input{background:#FFF;border:1px solid #CCC}input,select,textarea{font-family:Verdana,Arial,Helvetica,sans-serif;font-size:10px}input,select,textarea{border:1px solid gray}input.radio{border:1px none #000;background:transparent;vertical-align:middle}input.checkbox{border:1px none #000;background:transparent;vertical-align:middle}.input_noborder{border:0}#insert,#cancel,input.button,.updateButton{border:0;margin:0;padding:0;font-weight:bold;width:94px;height:26px;background:url(../img/buttons.png) 0 -26px;cursor:pointer;padding-bottom:2px;float:left}#insert{background:url(../img/buttons.png) 0 -52px}#cancel{background:url(../img/buttons.png) 0 0;float:right}a.pickcolor,a.browse{text-decoration:none}a.browse span{display:block;width:20px;height:18px;background:url(../img/icons.gif) -20px 0 no-repeat;border:1px solid #FFF;margin-left:1px}.mceOldBoxModel a.browse span{width:22px;height:20px}a.browse:hover span{border:1px solid #0a246a;background-color:#b2bbd0}a.browse span.disabled{border:1px solid white;opacity:.3;-ms-filter:'alpha(opacity=30)';filter:alpha(opacity=30)}a.browse:hover span.disabled{border:1px solid white;background-color:transparent}a.pickcolor span{display:block;width:20px;height:16px;background:url(../img/icons.gif) -840px 0;margin-left:2px}.mceOldBoxModel a.pickcolor span{width:21px;height:17px}a.pickcolor:hover span{background-color:#b2bbd0}table.charmap{border:1px solid #AAA;text-align:center}td.charmap,#charmap a{width:18px;height:18px;color:#000;border:1px solid #AAA;text-align:center;font-size:12px;vertical-align:middle;line-height:18px}#charmap a{display:block;color:#000;text-decoration:none;border:0}#charmap a:hover{background:#CCC;color:#2b6fb6}#charmap #codeN{font-size:10px;font-family:Arial,Helvetica,sans-serif;text-align:center}#charmap #codeV{font-size:40px;height:80px;border:1px solid #AAA;text-align:center}.wordWrapCode{vertical-align:middle;border:1px none #000;background:transparent}.mceActionPanel{margin-top:5px}.tabs{width:100%;height:18px;line-height:normal;background:url(../img/tabs.gif) repeat-x 0 -72px}.tabs ul{margin:0;padding:0;list-style:none}.tabs li{float:left;background:url(../img/tabs.gif) no-repeat 0 0;margin:0 2px 0 0;padding:0 0 0 10px;line-height:17px;height:18px;display:block}.tabs li.current{background:url(../img/tabs.gif) no-repeat 0 -18px;margin-right:2px}.tabs span{float:left;display:block;background:url(../img/tabs.gif) no-repeat right -36px;padding:0 10px 0 0}.tabs .current span{background:url(../img/tabs.gif) no-repeat right -54px}.tabs a{text-decoration:none;font-family:Verdana,Arial;font-size:10px}.tabs a:link,.tabs a:visited,.tabs a:hover{color:black}.panel_wrapper div.panel{display:none}.panel_wrapper div.current{display:block;width:100%;height:300px;overflow:visible}.panel_wrapper{border:1px solid #919b9c;border-top:0;padding:10px;padding-top:5px;clear:both;background:white}.column{float:left}.properties{width:100%}.properties .column2{text-align:left}h1,h2,h3,h4{color:#2b6fb6;margin:0;padding:0;padding-top:5px}h3{font-size:14px}.title{font-size:12px;font-weight:bold;color:#2b6fb6}#link .panel_wrapper,#link div.current{height:125px}#image .panel_wrapper,#image div.current{height:200px}#plugintable thead{font-weight:bold;background:#DDD}#plugintable,#about #plugintable td{border:1px solid #919b9c}#plugintable{width:96%;margin-top:10px}#pluginscontainer{height:290px;overflow:auto}#colorpicker #preview{display:inline-block;padding-left:40px;height:14px;border:1px solid black;margin-left:5px;margin-right:5px}#colorpicker #previewblock{position:relative;top:-3px;padding-left:5px;padding-top:0;display:inline}#colorpicker #preview_wrapper{text-align:center;padding-top:4px;white-space:nowrap}#colorpicker #colors{float:left;border:1px solid gray;cursor:crosshair}#colorpicker #light{border:1px solid gray;margin-left:5px;float:left;width:15px;height:150px;cursor:crosshair}#colorpicker #light div{overflow:hidden}#colorpicker .panel_wrapper div.current{height:175px}#colorpicker #namedcolors{width:150px}#colorpicker #namedcolors a{display:block;float:left;width:10px;height:10px;margin:1px 1px 0 0;overflow:hidden}#colorpicker #colornamecontainer{margin-top:5px}#colorpicker #picker_panel fieldset{margin:auto;width:325px}