const resizeLoginBox=function(){function e(){if(a){var f=.4375*b.clientHeight-c.clientHeight/2,g=(b.clientWidth-c.clientWidth)/2;c.style.top=f+"px";c.style.left=g+"px";d.style.width=b.clientWidth+"px";d.style.height=b.clientHeight+"px";d.style.top=-f+"px";d.style.left=-g+"px"}}const a=document.getElementsByTagName("body")[0],d=document.getElementById("bg"),c=document.getElementById("form-container"),b=document.getElementById("loading-mask")||a;a.className+=(0<a.className.length?" ":"")+navigator.sayswho.split(" ")[0]+
" "+navigator.sayswho.replace(" ","");var h=document.createElement("img");h.onload=function(){c.style.visibility="visible"};h.src=window.getComputedStyle(b,!1).backgroundImage.slice(4,-1).replace(/"/g,"");e();window.addEventListener("resize",e);return e}();
