.icon_desktopnotifications_settings{background-image:url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgaGVpZ2h0PSIxNiI+PHBhdGggZmlsbC1ydWxlPSJldmVub2RkIiBkPSJNOS4wMTUuOTdjMi4zNDguNDczIDQuMTMgMi41NTkgNC4xMyA1LjA0MSAwIDIuOTI2LS4yNzggNS4xMjcgMi42NjcgNy4wODZILjE4OGMyLjk1My0xLjk2NSAyLjY3NC00LjIwOCAyLjY3NC03LjA4NiAwLTIuNDk1IDEuODAxLTQuNTkgNC4xNjgtNS4wNDhDNy4xNjYtLjIyMSA4Ljg4Ny0uMjE1IDkuMDE1Ljk3em0xLjEwNyAxMy4yNDlhMi4xNTggMi4xNTggMCAwIDEtMi4xMjMgMS43NzMgMi4xNiAyLjE2IDAgMCAxLTIuMTIzLTEuNzczeiIgZmlsbD0iIzZkNmQ3MCIvPjwvc3ZnPg==);background-position:5px center}