.icon_mdm_settings{background-image:url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIxNiIgaGVpZ2h0PSIxNiIgdmlld0JveD0iMCAwIDExNC41NCAxMjIuODgiPjxwYXRoIGQ9Ik04NS4xIDR2MTQuNzJoLTUuNFY3LjFINnY5OC40Nmg0NC4wOHY5LjVhMTguMTMgMTguMTMgMCAwIDAgMS40MSA3bC4zNS44SDRhNCA0IDAgMCAxLTQtNFY0YTQgNCAwIDAgMSA0LTRoNzcuMTFhNCA0IDAgMCAxIDQgNHpNNzAuNTYgMzEuMDVIMTA1YTkuNTMgOS41MyAwIDAgMSA5LjUxIDkuNTF2NzIuODFhOS41MyA5LjUzIDAgMCAxLTkuNTEgOS41MUg3MC41NmE5LjUzIDkuNTMgMCAwIDEtOS41MS05LjUxVjQwLjU1YTkuNTEgOS41MSAwIDAgMSA5LjUxLTkuNXptLTYuMjMgNzguNjJoNDYuOTJWNDEuNDJINjQuMzN2NjguMjV6bTIzLjQ1IDNhMy45MSAzLjkxIDAgMSAxLTMuOTEgMy45MSAzLjkgMy45IDAgMCAxIDMuOTEtMy45MXptLTQ3LjE0LTMuMDdhNC41NyA0LjU3IDAgMSAxLTQuNTYgNC41NiA0LjU2IDQuNTYgMCAwIDEgNC41Ni00LjU2eiIgZmlsbD0iIzZkNmQ3MCIgZmlsbC1ydWxlPSJldmVub2RkIi8+PC9zdmc+Cg==) !important}.mdm-devicepanel.x-panel.tabpanel-container>.x-panel-bwrap>.x-panel-body{padding:0}.mdm-devicepanel.x-panel.tabpanel-container>.x-panel-bwrap>.x-panel-body>.x-tab-panel>.x-tab-panel-bwrap>.x-tab-panel-body{padding:0 !important}.mdm-display-name{font-weight:bold !important;padding:4px 0 2px 0}.mdm-device-panel{padding:5px 0 5px 10px}.mdm-field-sep{border-top-width:1px;border-top-style:solid;border-color:#e6e6e6}.mdm-synchronize-panel{border:0;margin-left:10px;padding:9px 0 0 0}.mdm-managesharedfolder-button{padding-right:10px}.mdm-create-tree-panel-item{margin:0 0 5px 0}.mdm-synchronize-shared-panel{padding-top:9px}.mdm-autherror-bar .x-btn-small{background:#af4236 !important;width:100%;text-align:center;box-shadow:none}.mdm-autherror-bar button{color:white !important;font-size:1.02em}.mdm-autherror-bar .x-btn-small:hover,.mdm-autherror-bar.x-btn-over>.x-btn-small,.mdm-autherror-bar.x-btn-over.x-btn-click>.x-btn-small,.mdm-autherror-bar.x-btn-over.x-btn-focus>.x-btn-small,.mdm-autherror-bar.x-btn-over.x-btn-click.x-btn-focus>.x-btn-small,.mdm-autherror-bar.x-btn-click>.x-btn-small{background:#b72b00 !important;box-shadow:none}