#printlist{width:100%;font-family:"Trebuchet MS",Arial,Helvetica,sans-serif;border-collapse:collapse}#printlist th{border:1px solid #ddd;padding:8px;font-size:13px;background-color:lightgrey;text-align:left;color:#222}#printlist td{padding:8px;font-size:11px;color:#222;border:1px solid #ddd}#printlist tr:nth-child(even){background-color:#f2f2f2}#printlistbottom{width:100%;padding-top:5px;color:#222;font-size:12px}#printlistbottom .right{float:right}