@import '_print';

	table {
		width: 100%;
		text-align: left;
  font-size: 11px;
  font-family: arial;
		border-collapse: collapse;
	}

		table th {
			font-weight: bold;
  padding: 2px 3px 2px 0px;
		}

		table td {
  padding: 2px 3px 2px 0px;
		}

/* date picker css */
.x-date-prevday a {
	color: transparent;
}
.x-date-nextday a {
	color: transparent;
		}
		.x-date-active a {
	color: black;
	text-decoration: none;
		}
.x-date-active {
	text-align: right;
		}
.x-date-left {
	display: none;
}
.x-date-right {
			display: none;
		}
		.x-btn button {
			background: none repeat scroll 0 0 transparent;
			border: 0 none;
		}

th.date-header {
	text-align: right;
    background-color: lightgrey;
	width: 50%;
		}
th.date-header-center {
	text-align: center;
    background-color: lightgrey;
	}

table.print-calendar {
	border: 1px solid black;
			border-collapse: collapse;
    height: 95%;
		}
.print-calendar th {
	border: 1px solid black;
		}

#top th {
	border: none;
}
table.inner {
	border: none;
		}

		.calendar-allday td {
			text-align: center;
	border: 1px solid black;
		}
		.calendar-normal td {
			text-align: left;
		}
td.calendar-item {
	width: 100%;
		}

.k-calendar-header {
	border: 1px solid black;
		}

		table.k-calendar-days {
	margin-top: 5px;

	th {
		border: 1px solid black;
				text-align: center;
		background-color: lightgrey;
		width: 14.28% !important;
	            }

				td {
		border: 1px solid #000;
					vertical-align: top;
					text-align: left;

					svg.k-appointment-status {
						position: absolute;
						display: inline-block;
						width: 7px;
						top: 0;
						left: 0;
						bottom: 0;
					}

					.k-appointment {
						font-size: 14px;
			width: 90%;
			height:20px;
			border-radius: 4px;
						line-height: 1.5;
			background-color:#FFF;
			position: absolute;
			left:2.5%;
						margin-bottom: 1px;
	}

		.k-overflow-indicator {
		display: inline-block;
			position: absolute;
			bottom: 0px;
			right: 0px;
			height: 10px;
		}
	}
}
