<?php return array(
    'root' => array(
        'name' => '__root__',
        'pretty_version' => 'dev-master',
        'version' => 'dev-master',
        'reference' => '785041accad7011ee3d096a5daf3a96d472497c4',
        'type' => 'library',
        'install_path' => __DIR__ . '/../../',
        'aliases' => array(),
        'dev' => true,
    ),
    'versions' => array(
        '__root__' => array(
            'pretty_version' => 'dev-master',
            'version' => 'dev-master',
            'reference' => '785041accad7011ee3d096a5daf3a96d472497c4',
            'type' => 'library',
            'install_path' => __DIR__ . '/../../',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/log' => array(
            'pretty_version' => '1.1.4',
            'version' => '1.1.4.0',
            'reference' => 'd49695b909c3b7628b6289db5479a1c204601f11',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/log',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'sabre/dav' => array(
            'pretty_version' => '4.4.0',
            'version' => '4.4.0.0',
            'reference' => 'b65362abc926520eda2c57e219f022a6c288069d',
            'type' => 'library',
            'install_path' => __DIR__ . '/../sabre/dav',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'sabre/event' => array(
            'pretty_version' => '5.1.4',
            'version' => '5.1.4.0',
            'reference' => 'd7da22897125d34d7eddf7977758191c06a74497',
            'type' => 'library',
            'install_path' => __DIR__ . '/../sabre/event',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'sabre/http' => array(
            'pretty_version' => '5.1.4',
            'version' => '5.1.4.0',
            'reference' => '84a83642c3b877da9d0e265fba7258157c0bdc37',
            'type' => 'library',
            'install_path' => __DIR__ . '/../sabre/http',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'sabre/uri' => array(
            'pretty_version' => '2.2.2',
            'version' => '2.2.2.0',
            'reference' => '7cb0f489578afad5006e85cd60f18ff33f2d440d',
            'type' => 'library',
            'install_path' => __DIR__ . '/../sabre/uri',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'sabre/vobject' => array(
            'pretty_version' => '4.4.2',
            'version' => '4.4.2.0',
            'reference' => 'b8a44ea1610d9b7012e42a7ff6f1cb66f0b757dd',
            'type' => 'library',
            'install_path' => __DIR__ . '/../sabre/vobject',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'sabre/xml' => array(
            'pretty_version' => '2.2.5',
            'version' => '2.2.5.0',
            'reference' => 'a6af111850e7536d200d9637c34885cd3c77a86c',
            'type' => 'library',
            'install_path' => __DIR__ . '/../sabre/xml',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
    ),
);
