.arrow_down_l, .x-panel-collapsed .x-tool.x-tool-toggle, td.sort-desc .x-grid3-sort-icon, .x-tree-lines .x-tree-elbow-end-minus, .x-tree-lines .x-tree-elbow-minus, .preview-header-collapse-plus, .zarafa-rules-sequence-down {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPSc5JyBoZWlnaHQ9JzUnIHZpZXdCb3g9JzAgMCA1LjQgMyc+PHBhdGggZD0nTTAgMGwyLjY5OSAzTDUuNCAwSDB6JyBmaWxsPScjNGY0YzRkJy8+PC9zdmc+) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.arrow_down_s, .x-btn .x-btn-split, .x-btn .x-btn-arrow, .x-form-trigger, .x-form-spinner-trigger {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPSc1JyBoZWlnaHQ9JzMnIHZpZXdCb3g9JzAgMCA1LjQgMyc+PHBhdGggZD0nTTAgMGwyLjY5OSAzTDUuNCAwSDB6JyBmaWxsPScjNGY0YzRkJy8+PC9zdmc+) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.arrow_down_s_white, .x-btn.x-btn.zarafa-action .x-btn-split, .x-btn.x-btn.zarafa-action .x-btn-arrow {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIGhlaWdodD0nMycgd2lkdGg9JzUnIHZpZXdCb3g9JzAgMCA1LjQgMyc+PHBhdGggZD0nTTAgMGwyLjY5OSAzTDUuNCAwSDB6JyBmaWxsPScjZmZmJy8+PC9zdmc+) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.arrow_left_l, .light-background .zarafa-calendar-tab-icon-merge, .x-date-left a, .x-date-mp-ybtn a.x-date-mp-prev, .x-tool.x-tool-toggle.x-tool-collapse-west, .x-layout-collapsed .x-tool.x-tool-toggle.x-tool-expand-east, .x-panel-header .x-tool.x-tool-collapse-west, .x-layout-collapsed .x-tool.x-tool-expand-east, .x-tab-scroller-left {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPSc1JyBoZWlnaHQ9JzknPjxwYXRoIGQ9J001IDB2OUwwIDQuNSA1IDB6JyBmaWxsPScjNGY0YzRkJy8+PC9zdmc+) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.arrow_left_l_white, .zarafa-calendar-tab-icon-merge {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPSc1JyBoZWlnaHQ9JzknPjxwYXRoIGQ9J001IDB2OUwwIDQuNSA1IDB6JyBmaWxsPScjZmZmJy8+PC9zdmc+) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.arrow_menubar, .zarafa-maintabbar-maintab-active button.x-btn-text, .zarafa-maintabbar-maintab-active.x-btn-over button.x-btn-text, .zarafa-maintabbar-maintab-active.x-btn-over.x-btn-click button.x-btn-text {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPScxMicgaGVpZ2h0PSc2Jz48cGF0aCBkPSdNNiAwTDAgNmgxMkw2IDB6JyBmaWxsPScjZmZmJy8+PC9zdmc+) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.arrow_right_l, .light-background .zarafa-calendar-tab-icon-separate, .x-date-right a, .x-date-mp-ybtn a, .x-tree-lines .x-tree-elbow-plus, .x-tree-lines .x-tree-elbow-end-plus, .x-menu-item-arrow, .x-tool.x-tool-toggle.x-tool-collapse-east, .x-layout-collapsed .x-tool.x-tool-toggle.x-tool-expand-west, .x-panel-header .x-tool.x-tool-collapse-east, .x-layout-collapsed .x-tool.x-tool-expand-west, .x-tab-scroller-right {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPSc1JyBoZWlnaHQ9JzknPjxwYXRoIGQ9J00wIDB2OWw1LTQuNUwwIDB6JyBmaWxsPScjNGY0YzRkJy8+PC9zdmc+) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.arrow_right_l_white, .zarafa-calendar-tab-icon-separate {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPSc1JyBoZWlnaHQ9JzknPjxwYXRoIGQ9J00wIDB2OWw1LTQuNUwwIDB6JyBmaWxsPScjZmZmJy8+PC9zdmc+) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.arrow_up_l, .x-tool.x-tool-toggle, td.sort-asc .x-grid3-sort-icon, .preview-header-collapse-minus, .zarafa-rules-sequence-up {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIGhlaWdodD0nNScgd2lkdGg9JzknIHZpZXdCb3g9JzAgMCA1LjQgMyc+PHBhdGggZD0nTTAgM2wyLjY5OS0zTDUuNCAzSDB6JyBmaWxsPScjNGY0YzRkJy8+PC9zdmc+) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.arrow_updown, .x-form-field-wrap .x-form-spinner-trigger {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIGhlaWdodD0nMTQnIHdpZHRoPSc1JyBmaWxsPScjNGY0YzRkJz48cGF0aCBkPSdNMCAxMS4ybDIuNDk5IDIuNzc4TDUgMTEuMTc4SDB6TTAgM0wyLjQ5OS4yMjIgNSAzSDB6Jy8+PC9zdmc+) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.icon_cross, .icon_remove, .x-zarafa-boxfield-item-close, .x-zarafa-boxfield-suggestion-cross, .light-background .zarafa-calendar-tab-icon-close, .x-freebusy-userrow-remove, .x-tab-strip .x-tab-strip-closable a.x-tab-strip-close {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPSc3JyBoZWlnaHQ9JzcnPjxwYXRoIGQ9J00uNzIxLjAyM0wuMDE2LjczIDIuNzkzIDMuNS4wMTYgNi4yN2wuNzA1LjcwN0wzLjUgNC4yMDVsMi43NzkgMi43NzEuNzA1LS43MDdMNC4yMDcgMy41IDYuOTg0LjczIDYuMjc5LjAyMyAzLjUgMi43OTUuNzIxLjAyM3onIGZpbGw9JyM0ZjRjNGQnLz48L3N2Zz4=) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.icon_cross_large, .x-tool-close {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPScxMCcgaGVpZ2h0PScxMCc+PHBhdGggZD0nTS43MDkgMEwwIC43MDkgNC4yOTEgNSAwIDkuMjkxLjcwOSAxMCA1IDUuNzA5IDkuMjkxIDEwIDEwIDkuMjkxIDUuNzA5IDUgMTAgLjcwOSA5LjI5MSAwIDUgNC4yOTEuNzA5IDB6JyBmaWxsPScjNGY0YzRkJy8+PC9zdmc+) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.icon_cross_white, .zarafa-calendar-tab-icon-close, .k-category-close {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPSc3JyBoZWlnaHQ9JzcnPjxwYXRoIGQ9J00uNzIxLjAyM0wuMDE2LjczIDIuNzkzIDMuNS4wMTYgNi4yN2wuNzA1LjcwN0wzLjUgNC4yMDVsMi43NzkgMi43NzEuNzA1LS43MDdMNC4yMDcgMy41IDYuOTg0LjczIDYuMjc5LjAyMyAzLjUgMi43OTUuNzIxLjAyM3onIGZpbGw9JyNmZmYnLz48L3N2Zz4=) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.icon_tools_cogwheel, .x-tool.x-tool-gear {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPScxMycgaGVpZ2h0PScxMycgdmlld0JveD0nMCAwIDE2IDE2Jz48cGF0aCBkPSdNNi4yNTIuOTk4bC0uMTU0IDEuODQ0YTUuNSA1LjUgMCAwIDAtMS42MDcuOTNsLTEuNjc2LS43ODktMS43NSAzLjAzMUwyLjU4NiA3LjA3YTUuNSA1LjUgMCAwIDAtLjA4NC45MjggNS41IDUuNSAwIDAgMCAuMDguOTMyTDEuMDY0IDkuOTgybDEuNzUgMy4wMzMgMS42NzItLjc4N2E1LjUgNS41IDAgMCAwIDEuNjExLjkyNGwuMTUgMS44NWgzLjVsLjE1NC0xLjg0NGE1LjUgNS41IDAgMCAwIDEuNjA3LS45M2wxLjY3Ni43ODkgMS43NS0zLjAzMy0xLjUyMS0xLjA1N0E1LjUgNS41IDAgMCAwIDEzLjQ5OCA4YTUuNSA1LjUgMCAwIDAtLjA4LS45MzJsMS41MTgtMS4wNTMtMS43NS0zLjAzMS0xLjY3Mi43ODdhNS41IDUuNSAwIDAgMC0xLjYxMS0uOTI0bC0uMTUtMS44NWgtMy41ek04IDVhMyAzIDAgMSAxIDAgNiAzIDMgMCAxIDEgMC02eicgZmlsbD0nIzRmNGM0ZCcvPjwvc3ZnPg==) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.icon_tools_minimize, .x-tool.x-tool-minimize {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPScxMCcgaGVpZ2h0PScxMCcgdmlld0JveD0nMCAwIDEzIDEzJz48cGF0aCBkPSdNLjUgMTBhLjUuNSAwIDEgMCAwIDFoMTJhLjUuNSAwIDEgMCAwLTFILjV6JyBmaWxsPScjNGY0YzRkJy8+PC9zdmc+) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.line_arrow_down_l {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPSc5JyBoZWlnaHQ9JzQ1LjAxNCcgdmlld0JveD0nMCAwIDIuMzgxIDExLjkxJz48cGF0aCBkPSdNMS4xOTEgMTEuOTFWLjU2NycgZmlsbD0nbm9uZScgc3Ryb2tlLXdpZHRoPScuMjY1JyBzdHJva2U9JyM0ZjRjNGQnLz48cGF0aCBkPSdNMCAwbDEuMTkgMS4zMjNMMi4zODEgMHonIGZpbGw9JyM0ZjRjNGQnLz48L3N2Zz4=) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.line_arrow_right_l {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPScxMCcgaGVpZ2h0PSc5JyB2aWV3Qm94PScwIDAgMi42NDYgMi4zODEnPjxwYXRoIGQ9J00wIDEuMTloMS41ODgnIGZpbGw9J25vbmUnIHN0cm9rZS13aWR0aD0nLjI2NScgc3Ryb2tlPScjNGY0YzRkJy8+PHBhdGggZD0nTTEuMzIzIDB2Mi4zODFsMS4zMjMtMS4xOTF6JyBmaWxsPScjNGY0YzRkJy8+PC9zdmc+) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.line_vertical {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPScxJyBoZWlnaHQ9JzUwMCcgdmlld0JveD0nMCAwIDAuMjY1IDEzMi4yOSc+PHBhdGggZD0nTS4xMzIgMHYxMzIuMjknIGZpbGw9J25vbmUnIHN0cm9rZT0nIzRmNGM0ZCcgc3Ryb2tlLXdpZHRoPScuMjY1Jy8+PC9zdmc+) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.minimize_button {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPScxMycgaGVpZ2h0PScxMyc+PHBhdGggZD0nTS41IDZhLjUuNSAwIDEgMCAwIDFoMTJhLjUuNSAwIDEgMCAwLTFILjV6JyBmaWxsPScjNGY0YzRkJy8+PC9zdmc+) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.plus, .x-tab-add, .x-tool.x-tool-plus, .zarafa-rules-add, .x-zarafa-boxfield-item-expand {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPScxMycgaGVpZ2h0PScxMyc+PHBhdGggZD0nTTYuNSAwYS41LjUgMCAwIDAtLjUuNVY2SC41YS41LjUgMCAxIDAgMCAxSDZ2NS41YS41LjUgMCAxIDAgMSAwVjdoNS41YS41LjUgMCAxIDAgMC0xSDdWLjVhLjUuNSAwIDAgMC0uNS0uNXonIGZpbGw9JyM0ZjRjNGQnLz48L3N2Zz4=) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.triangle_left, .x-tbar-page-prev {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPScxNicgaGVpZ2h0PScxNic+PHBhdGggZD0nTTExLjQ4NCAzYS41LjUgMCAwIDAtLjI1NC4wOGwtNyA0LjVhLjUuNSAwIDAgMCAwIC44NGw3IDQuNWEuNS41IDAgMCAwIC43Ny0uNDJ2LTlhLjUuNSAwIDAgMC0uNTE2LS41eicgZmlsbD0nIzRmNGM0ZCcvPjwvc3ZnPg==) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.triangle_left_outline, .x-item-disabled .x-tbar-page-prev {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPScxNicgaGVpZ2h0PScxNic+PHBhdGggZD0nTTExLjQ4NCAzYS41LjUgMCAwIDAtLjI1NC4wOGwtNyA0LjVhLjUuNSAwIDAgMCAwIC44NGw3IDQuNWEuNS41IDAgMCAwIC43Ny0uNDJ2LTlhLjUuNSAwIDAgMC0uNTE2LS41ek0xMSA0LjQxNHY3LjE3Mkw1LjQyMiA4IDExIDQuNDE0eicgZmlsbD0nIzRmNGM0ZCcvPjwvc3ZnPg==) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.triangle_left_stop, .x-tbar-page-first {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPScxNicgaGVpZ2h0PScxNic+PHBhdGggZD0nTTQgM2ExIDEgMCAwIDAtMSAxdjhhMSAxIDAgMCAwIDEgMWgxYTEgMSAwIDAgMCAxLTFWOC45MTRsNi4yMyA0LjAwNmEuNS41IDAgMCAwIC43Ny0uNDJ2LTlhLjUuNSAwIDAgMC0uNzctLjQyTDYgNy4wODZWNGExIDEgMCAwIDAtMS0xeicgZmlsbD0nIzRmNGM0ZCcvPjwvc3ZnPg==) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.triangle_left_stop_outline, .x-item-disabled .x-tbar-page-first {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPScxNicgaGVpZ2h0PScxNic+PHBhdGggZD0nTTQgM2ExIDEgMCAwIDAtMSAxdjhhMSAxIDAgMCAwIDEgMWgxYTEgMSAwIDAgMCAxLTFWOC45MTRsNi4yMyA0LjAwNmEuNS41IDAgMCAwIC43Ny0uNDJ2LTlhLjUuNSAwIDAgMC0uNzctLjQyTDYgNy4wODZWNGExIDEgMCAwIDAtMS0xSDR6bTAgLjgzaDFWMTJINFYzLjgzem04IC41ODR2Ny4xNzJMNi40MjIgOCAxMiA0LjQxNHonIGZpbGw9JyM0ZjRjNGQnLz48L3N2Zz4=) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.triangle_right, .x-tbar-page-next {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPScxNicgaGVpZ2h0PScxNic+PHBhdGggZD0nTTQuNTE4IDNhLjUuNSAwIDAgMSAuMjU0LjA4bDcgNC41YS41LjUgMCAwIDEgMCAuODRsLTcgNC41YS41LjUgMCAwIDEtLjc3LS40MnYtOWEuNS41IDAgMCAxIC41MTYtLjV6JyBmaWxsPScjNGY0YzRkJy8+PC9zdmc+) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.triangle_right_outline, .x-item-disabled .x-tbar-page-next {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPScxNicgaGVpZ2h0PScxNic+PHBhdGggZD0nTTQuNTE4IDNhLjUuNSAwIDAgMSAuMjU0LjA4bDcgNC41YS41LjUgMCAwIDEgMCAuODRsLTcgNC41YS41LjUgMCAwIDEtLjc3LS40MnYtOWEuNS41IDAgMCAxIC41MTYtLjV6bS40ODQgMS40MTR2Ny4xNzJMMTAuNTggOCA1LjAwMiA0LjQxNHonIGZpbGw9JyM0ZjRjNGQnLz48L3N2Zz4=) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.triangle_right_stop, .x-tbar-page-last {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPScxNicgaGVpZ2h0PScxNic+PHBhdGggZD0nTTEyIDNhMSAxIDAgMCAxIDEgMXY4YTEgMSAwIDAgMS0xIDFoLTFhMSAxIDAgMCAxLTEtMVY4LjkxNEwzLjc3IDEyLjkyQS41LjUgMCAwIDEgMyAxMi41di05YS41LjUgMCAwIDEgLjc3LS40MkwxMCA3LjA4NlY0YTEgMSAwIDAgMSAxLTF6JyBmaWxsPScjNGY0YzRkJy8+PC9zdmc+) !important; background-repeat: no-repeat !important; background-position: center center !important;}
.triangle_right_stop_outline, .x-item-disabled .x-tbar-page-next {background-image: url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0naHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmcnIHdpZHRoPScxNicgaGVpZ2h0PScxNic+PHBhdGggZD0nTTEyIDNhMSAxIDAgMCAxIDEgMXY4YTEgMSAwIDAgMS0xIDFoLTFhMSAxIDAgMCAxLTEtMVY4LjkxNEwzLjc3IDEyLjkyQS41LjUgMCAwIDEgMyAxMi41di05YS41LjUgMCAwIDEgLjc3LS40MkwxMCA3LjA4NlY0YTEgMSAwIDAgMSAxLTFoMXptMCAuODNoLTFWMTJoMVYzLjgzem0tOCAuNTg0djcuMTcyTDkuNTc4IDggNCA0LjQxNHonIGZpbGw9JyM0ZjRjNGQnLz48L3N2Zz4=) !important; background-repeat: no-repeat !important; background-position: center center !important;}