@page{size:auto;margin:1.4cm}@media print{thead{display:table-header-group}#pagemargin{min-height:100%;margin:0}}html,body,div,dl,dt,dd,h1,h2,h3,h4,h5,h6,pre,form,fieldset,input,p,blockquote,th,td{margin:0;padding:0}img,body,html{border:0}html,body{width:100%;height:100%}address,caption,cite,code,dfn,th,var{font-style:normal;font-weight:normal}caption,th{text-align:left}h1,h2,h3,h4,h5,h6{font-size:100%}q:before,q:after{content:''}.nowrap{white-space:nowrap}.wrap{white-space:normal}.fullwidth{width:100%}table{width:100%;text-align:left;font-size:11px;font-family:Arial,Helvetica,sans-serif;border-collapse:collapse}table th{font-weight:bold;padding:2px 3px 2px 0}table td{padding:2px 3px 2px 0}.x-date-prevday a{color:transparent}.x-date-nextday a{color:transparent}.x-date-active a{color:black;text-decoration:none}.x-date-active{text-align:right}.x-date-left{display:none}.x-date-right{display:none}.x-btn button{background:none repeat scroll 0 0 transparent;border:0 none}th.date-header{text-align:right;background-color:lightgrey;width:50%}th.date-header-center{text-align:center;background-color:lightgrey}table.print-calendar{border:1px solid black;border-collapse:collapse;height:95%}.print-calendar th{border:1px solid black}#top th{border:0}table.inner{border:0}.calendar-allday td{text-align:center;border:1px solid black}.calendar-normal td{text-align:left}td.calendar-item{width:100%}.k-calendar-header{border:1px solid black}table.k-calendar-days{margin-top:5px}table.k-calendar-days th{border:1px solid black;text-align:center;background-color:lightgrey;width:14.28% !important}table.k-calendar-days td{border:1px solid #000;vertical-align:top;text-align:left}table.k-calendar-days td svg.k-appointment-status{position:absolute;display:inline-block;width:7px;top:0;left:0;bottom:0}table.k-calendar-days td .k-appointment{font-size:14px;width:90%;height:20px;border-radius:4px;line-height:1.5;background-color:#FFF;position:absolute;left:2.5%;margin-bottom:1px}table.k-calendar-days td .k-overflow-indicator{display:inline-block;position:absolute;bottom:0;right:0;height:10px}